<?php

namespace Bydedi\Shipping\Chronopost;

use Bydedi\Shipping\Chronopost\Exception\ChronopostException;
use ChronopostServiceShipping;
use ChronopostStructCustomerValue;
use ChronopostStructEsdValue;
use ChronopostStructEsdWithRefClientValue;
use ChronopostStructHeaderValue;
use ChronopostStructShipperValue;
use ChronopostStructShipping;
use ChronopostStructSkybillParamsValue;
use ChronopostStructSkybillValue;
use ChronopostWsdlClass;

/**
 * Class ShippingService
 * @package Bydedi\Shipping\Chronopost
 */
class ShippingService
{
    const WSDL = 'https://ws.chronopost.fr/shipping-cxf/ShippingServiceWS?wsdl';

    /**
     * @var string
     */
    protected $login;

    /**
     * @var string
     */
    protected $pass;

    /**
     * @var string
     */
    protected $ws_pass;

    /**
     * @var string
     */
    protected $dir;

    /**
     * @var array
     */
    protected $shipper_infos;

    /**
     * @var ChronopostServiceShipping
     */
    protected $chronopostServiceShipping;

    /**
     * ShippingService constructor.
     *
     * @param $config
     *
     * @throws ChronopostException
     */
    public function __construct($config)
    {
        $this->login = $config['login'];
        $this->pass = $config['pass'];
        $this->ws_pass = $config['ws_pass'];
        $this->dir = $config['dir'];
        $this->shipper_infos = $config['shipper'];

        $this->chronopostServiceShipping = new ChronopostServiceShipping($this->getSoapConfig());

        if (!is_dir($this->dir) && !mkdir($this->dir, 0775, true)) {
            throw new ChronopostException(sprintf('The dir %s is not writable', $this->dir));
        }
    }

    /**
     * @param Sticker $sticker
     *
     * @return Sticker
     *
     * @throws ChronopostException
     */
    public function generate(Sticker $sticker)
    {
        $shipper = $sticker->getShipper();
        if (empty($shipper)) {
            $shipper = $this->createDefaultShipper();
        }

        $customer = $sticker->getCustomer();
        if (empty($customer)) {
            $customer = $this->createDefaultCustomer();
        }

        $header = new ChronopostStructHeaderValue($this->login, 'CHRFR');

        $date = date('Y-m-d');

        $skybill = new ChronopostStructSkybillValue(
            "bulkNumber",
            $sticker->getCurrency(),
            "0",
            "",
            "",
            "",
            "",
            "",
            $sticker->getCurrency(),
            "0",
            "DC",
            $sticker->getCurrency(),
            "0",
            "MAR",
            $sticker->getCurrency(),
            $sticker->getPortValue(),
            $sticker->getProductCode(),
            0,
            date('Y-m-d\TH:i:sP', strtotime($date . 'T14:30:00Z')),
            "10",
            "",
            (int)$sticker->getWeight()/1000,
            "KGM"
        );

        $skybillParams = new ChronopostStructSkybillParamsValue("PDF");

        $refEsdClient = new ChronopostStructEsdValue(
            date('Y-m-d\TH:i:sP', strtotime($date . 'T17:30:00Z')),
            10,
            10,
            date('Y-m-d\TH:i:sP', strtotime($date . 'T14:30:00Z')),
            "1",
            "",
            "",
            "",
            10
        );

        $esdRefClient = new ChronopostStructEsdWithRefClientValue(0, 0);
        $esdRefClient->setHeight($refEsdClient->getHeight());
        $esdRefClient->setWidth($refEsdClient->getWidth());
        $esdRefClient->setLength($refEsdClient->getLength());

        //objet final
        $theMethod = new ChronopostStructShipping(
            $esdRefClient,
            $header,
            $shipper,
            $customer,
            $sticker->getRecipient(),
            $sticker->getRef(),
            $skybill,
            $skybillParams,
            $this->ws_pass
        );

        //appel de la fonction

        $result = $this->chronopostServiceShipping->shipping($theMethod);
        if (!$result) {
            throw new ChronopostException($this->chronopostServiceShipping->getLastError());
        }

        if (!empty($this->chronopostServiceShipping->getResult()->return->return->errorCode)) {
            throw new ChronopostException(
                sprintf(
                    '%s : %s',
                    $this->chronopostServiceShipping->getResult()->return->return->errorCode,
                    $this->chronopostServiceShipping->getResult()->return->return->errorMessage
                )
            );
        }

        if (empty($this->chronopostServiceShipping->getResult()->return->return->skybill)) {
            throw new ChronopostException($this->chronopostServiceShipping->getResult());
        }

        $filename = sprintf(
            '%s.pdf',
            $this->chronopostServiceShipping->getResult()->return->return->skybillNumber
        );

        $filePath = $this->dir . $filename;
        $contentPDF = $this->chronopostServiceShipping->getResult()->return->return->skybill;

        file_put_contents($filePath, $contentPDF);

        $sticker
            ->setUrlPDF($filePath)
            ->setTracking($this->chronopostServiceShipping->getResult()->return->return->skybillNumber)
        ;

        return $sticker;
    }

    public function expedie($order_id = null)
    {
        // TODO : expedie la commande passé en parametre,
        // sinon toutes les commandes en cours de prepa avec un numero de colis
    }

    /**
     * @return array
     */
    protected function getSoapConfig()
    {
        return [
            ChronopostWsdlClass::WSDL_URL => self::WSDL,
            ChronopostWsdlClass::WSDL_CACHE_WSDL => WSDL_CACHE_NONE,
            ChronopostWsdlClass::WSDL_TRACE => true,
            ChronopostWsdlClass::WSDL_LOGIN => $this->login,
            ChronopostWsdlClass::WSDL_PASSWD => $this->ws_pass,
        ];
    }

    /**
     * @return ChronopostStructShipperValue
     */
    protected function createDefaultShipper()
    {
        return new ChronopostStructShipperValue(
            $this->shipper_infos['adresse_rue'],
            $this->shipper_infos['adresse_rue2'],
            $this->shipper_infos['adresse_ville'],
            $this->shipper_infos['civilite'],
            $this->shipper_infos['nom'],
            $this->shipper_infos['pays_code'],
            $this->shipper_infos['pays_nom'],
            $this->shipper_infos['email'],
            $this->shipper_infos['telephone'],
            $this->shipper_infos['societe'],
            "",
            $this->shipper_infos['telephone'],
            0,
            $this->shipper_infos['adresse_cp']
        );
    }

    protected function createDefaultCustomer()
    {
        return new ChronopostStructCustomerValue(
            $this->shipper_infos['adresse_rue'],
            $this->shipper_infos['adresse_rue2'],
            $this->shipper_infos['adresse_ville'],
            $this->shipper_infos['civilite'],
            $this->shipper_infos['nom'],
            $this->shipper_infos['pays_code'],
            $this->shipper_infos['pays_nom'],
            $this->shipper_infos['email'],
            $this->shipper_infos['telephone'],
            $this->shipper_infos['societe'],
            "",
            $this->shipper_infos['telephone'],
            0,
            $this->shipper_infos['adresse_cp'],
            'Y'
        );
    }
}
